/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "practico2b.h"
#include <QPainter>
#include <QMouseEvent>
#include <QPoint>
#include <QColor>
#include <QMessageBox>
#include <QDebug>



Practico2b::Practico2b()
{
	setBaseSize(800, 600);
	previousPointExists = false;
	centerX = width() / 2;
	centerY = height()  / 2;
	windowSize = 1024;
	squareSize = 3*windowSize/5;
	windowSizeRelationship = (double)windowSize / (double)qMin(width(), height());
}

Practico2b::~Practico2b()
{
	// Empty the lines list
	qDeleteAll(linesList.begin(), linesList.end());
	linesList.clear();
}

void Practico2b::draw(QPainter * painter)
{
	for(int i=0; i < linesList.size(); i++)
	{
		painter->setPen(linesList[i]->getColor());
		painter->drawLine(linesList[i]->getFirstPoint(),
		                  linesList[i]->getSecondPoint());
	}
}

void Practico2b::paintEvent(QPaintEvent *)
{
	QPainter painter(this);
	painter.setRenderHint(QPainter::Antialiasing);
	painter.setBackground(QBrush(QColor(255, 255, 255)));
	painter.fillRect(0, 0, width(), height(), QBrush(QColor(255, 255, 255)));

	int side = qMin(width(), height());

	painter.setViewport((width()-side)/2, (height()-side)/2, side, side);
	painter.setWindow(-windowSize/2, -windowSize/2, windowSize, windowSize);

	centerX = width() / 2;
	centerY = height()  / 2;
	windowSizeRelationship = (double)windowSize / (double)side;

	// Create a rectangle
	painter.drawRect(-squareSize/2, -squareSize/2, squareSize, squareSize);

	draw(&painter);
}


void Practico2b::mousePressEvent(QMouseEvent * event)
{
	// We check that the left button has been pressed
	if(event->buttons() & Qt::LeftButton)
	{
// 		QString xCoord = QString(" %1 ").arg((event->x() - centerX)*windowSizeRelationship);
// 		QString yCoord = QString(" %1 ").arg((event->y() - centerY)*windowSizeRelationship);
// 		QMessageBox::information(this,tr("Práctico1"),
// 		                         tr("El punto donde se presionó es ") +
// 		                         xCoord + yCoord);

		if(!previousPointExists)
		{
			// Get the click point, center it
			firstPoint = event->pos() - QPoint(centerX, centerY);
			// Apply the relatonship of the window
			firstPoint *= windowSizeRelationship;

			previousPointExists = true;
		}
		else
		{
			secondPoint = event->pos() - QPoint(centerX, centerY);
			secondPoint *= windowSizeRelationship;

			// Here we call the funtion in progress
			// TODO
			Line * line = new Line(firstPoint, secondPoint, QColor("black"));
			setLineTags(line);
			clipLine(line);

			update();

			// Finally, reset the input
			previousPointExists = false;
		}
	}
}

void Practico2b::setLineTags(Line * theLine)
{
	theLine->setFirstPointTag(setPointTag(theLine->getFirstPoint()));
	theLine->setSecondPointTag(setPointTag(theLine->getSecondPoint()));
}

int Practico2b::setPointTag(const QPoint & point)
{
	int tag = 0;

	// First check in x direction
	if(point.x() > (squareSize/2))
		tag = tag | 2;
	else if(point.x() < (-squareSize/2))
		tag = tag | 1;

	// Next in y direction
	if(point.y() > (squareSize/2))
		tag = tag | 4;
	else if(point.y() < (-squareSize/2))
		tag = tag | 8;

	return tag;
}

void Practico2b::clipLine(Line * line)
{
	double intersectionY = 0.0;
	double intersectionX = 0.0;

	// Check where the line is
	if((line->getFirstPointTag() | line->getSecondPointTag()) == 0)
	{
		// Entirely valid
		linesList.append(line);
		return;
	}
	else if((line->getFirstPointTag() & line->getSecondPointTag()) != 0)
	{
		// Totally invisible
		line->setColor(QColor(Qt::lightGray));
		linesList.append(line);
		return;
	}

	/*
		Now we have lines that will clip. We will consider that the first point is
		outside the drawing area. If that's not the case, we switch the points.
	*/
	if(line->getFirstPointTag() == 0)
	{
		QPoint temp = line->getFirstPoint();
		line->setFirstPoint(line->getSecondPoint());
		line->setSecondPoint(temp);
		setLineTags(line);
	}

	if((line->getFirstPointTag() == 9) ||
	        (line->getFirstPointTag() == 1) ||
	        (line->getFirstPointTag() == 5))
	{
		// We consider the left cases, we would surely split the line
		Line * newLine = new Line(line->getFirstPoint(), QPoint(0, 0),
		                          QColor(Qt::red));

		intersectionY = ((-squareSize/2) - line->getFirstPoint().x());
		intersectionY /= (line->getSecondPoint().x()  - line->getFirstPoint().x());
		intersectionY *= (line->getSecondPoint().y() - line->getFirstPoint().y());
		intersectionY += line->getFirstPoint().y();

		newLine->setSecondPoint(QPoint(-squareSize/2, (int)intersectionY));
		/*
		This step is unnecesary for drawing, but if someone modifies the code, it
		would really help him / her.
		*/
		setLineTags(newLine);
		linesList.append(newLine);

		// Now we switch the point in the original line.
		line->setFirstPoint(line->getSecondPoint());
		line->setSecondPoint(newLine->getSecondPoint());
		setLineTags(line);

		// Finally, we apply this algorythm to the remaining line.
		clipLine(line);
	}
	else if((line->getFirstPointTag() == 10) ||
	        (line->getFirstPointTag() == 2) ||
	        (line->getFirstPointTag() == 6))
	{
		// We consider the right cases, we would surely split the line
		Line * newLine = new Line(line->getFirstPoint(), QPoint(0, 0),
		                          QColor(Qt::magenta));

		intersectionY = ((squareSize/2) - line->getFirstPoint().x());
		intersectionY /= (line->getSecondPoint().x()  - line->getFirstPoint().x());
		intersectionY *= (line->getSecondPoint().y() - line->getFirstPoint().y());
		intersectionY += line->getFirstPoint().y();

		newLine->setSecondPoint(QPoint(squareSize/2, (int)intersectionY));
		/*
		This step is unnecesary for drawing, but if someone modifies the code, it
		would really help him / her.
		*/
		setLineTags(newLine);
		linesList.append(newLine);

		// Now we switch the point in the original line.
		line->setFirstPoint(line->getSecondPoint());
		line->setSecondPoint(newLine->getSecondPoint());
		setLineTags(line);

		// Finally, we apply this algorythm to the remaining line.
		clipLine(line);
	}
	else if(line->getFirstPointTag() == 8)
	{
		// We consider the upper case, we would surely split the line
		Line * newLine = new Line(line->getFirstPoint(), QPoint(0, 0),
		                          QColor(Qt::blue));

		intersectionX = ((-squareSize/2) - line->getFirstPoint().y());
		intersectionX /= (line->getSecondPoint().y()  - line->getFirstPoint().y());
		intersectionX *= (line->getSecondPoint().x() - line->getFirstPoint().x());
		intersectionX += line->getFirstPoint().x();

		newLine->setSecondPoint(QPoint((int)intersectionX, -squareSize/2));
		/*
		This step is unnecesary for drawing, but if someone modifies the code, it
		would really help him / her.
		*/
		setLineTags(newLine);
		linesList.append(newLine);

		// Now we switch the point in the original line.
		line->setFirstPoint(line->getSecondPoint());
		line->setSecondPoint(newLine->getSecondPoint());
		setLineTags(line);

		// Finally, we apply this algorythm to the remaining line.
		clipLine(line);
	}
	else if(line->getFirstPointTag() == 4)
	{
		// We consider the lower case, we would surely split the line
		Line * newLine = new Line(line->getFirstPoint(), QPoint(0, 0),
		                          QColor(Qt::green));

		intersectionX = ((squareSize/2) - line->getFirstPoint().y());
		intersectionX /= (line->getSecondPoint().y()  - line->getFirstPoint().y());
		intersectionX *= (line->getSecondPoint().x() - line->getFirstPoint().x());
		intersectionX += line->getFirstPoint().x();

		newLine->setSecondPoint(QPoint((int)intersectionX, squareSize/2));
		/*
		This step is unnecesary for drawing, but if someone modifies the code, it
		would really help him / her.
		*/
		setLineTags(newLine);
		linesList.append(newLine);

		// Now we switch the point in the original line.
		line->setFirstPoint(line->getSecondPoint());
		line->setSecondPoint(newLine->getSecondPoint());
		setLineTags(line);

		// Finally, we apply this algorythm to the remaining line.
		clipLine(line);
	}
	// If everything else fails
	else
	{
		qDebug() << line->getFirstPoint() << line->getFirstPointTag();
		qDebug() << line->getSecondPoint() << line->getSecondPointTag();
		delete line;
	}

}

