/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PRACTICO2B_H
#define PRACTICO2B_H

#include "line.h"
#include <QMainWindow>
#include <QList>

class QPainter;
class QPaintEvent;
class QPoint;

/// Practico2b is the main class for the Practico 2 of PSG
/**
	This class will draw a rectangle and lines, to show how
	clipping algorythms work.
*/

class Practico2b:public QMainWindow
{
	Q_OBJECT

public:
	Practico2b();
	/// the destructor
	/**
		Cleans linesList, calling delete in all it's components and
		clearing the list.
	*/
	~Practico2b();

	/// Draw the scene
	void draw(QPainter * painter);

protected:
	void paintEvent(QPaintEvent * event);
	void mousePressEvent(QMouseEvent * event);

private:
	/// Sets the Line's tags
	/**
		This function calls setPointTag twice, one for each line's extreme.
	*/
	void setLineTags(Line * theLine);

	/// Set the point's tag
	int setPointTag(const QPoint & point);

	/// Clasify and "clip" the lines
	/**
		This function will clasify a line, and if necessary, "clip it" by dividing
		it and changing it's colors accordingly. \n
		It will also put the lines in the list.
	*/
	void clipLine(Line * line);

	/// Check if there is a previous point ready
	bool previousPointExists;

	/// First assigned point
	QPoint firstPoint;

	/// Second assigned point
	QPoint secondPoint;

	/// The center of the x coordinate
	int centerX;
	/// The center of the y coordinate
	int centerY;

	/// The size of the windows
	int windowSize;

	/// The size of the square
	int squareSize;

	/// Window size / real size relationship
	/**
		This relationship is use d to obtain the value of the clicking point
		of the mouse. \n
		Let: \n
		* CP: click point relative to the widget \n
		* GP: graphic point, point reltive to the window \n
		* RS: real size, the minimum between the height and width of the widget \n
		Then:\n
		GP = windowSize * CP / RS \n
		Then the relationship windowSizeRelationship would be: \n
		windowSizeRelationship = windowSize / RS
	*/
	double windowSizeRelationship;

	/// A list of lines
	QList < Line * > linesList;
};

#endif
